*********************************************************************
* expservo.asm                                                      *
*                                                                   *
* Handy Board Expansion Board servo drivers (Exp Bd rev 3)          *
*                                                                   *
* Fred Martin        fredm@media.mit.edu           12 June 1998     *
*********************************************************************

* 6811 definitions
REGBASE	equ	$1000
PORTA	equ	$1000	; Port A data register
PORTC	equ	$1003	; Port C 
CFORC	equ	$100B	; Timer Compare Force Register
TCNT	equ	$100E	; Timer Count Register
TOC3	equ	$101A	; Timer Output Compare register 3
TCTL1	equ	$1020	; Timer Control register 1
TCTL2	equ	$1021	; Timer Control register 2
TMSK1	equ	$1022	; main Timer interrupt Mask register 1
TFLG1	equ	$1023	; main Timer interrupt Flag register 1
TMSK2	equ	$1024	; misc Timer interrupt Mask register 2
TFLG2	equ	$1025	; misc Timer interrupt Flag register 2

TOC3INT	equ	$bfe4	; timer output 3 interrupt vector

* latch address and storage
EXPLATCH	equ	$4000	; HW address of servo, analog latch
EXPVAL		equ	$31	; RAM address for exp latch value

* servo constants
MIN_PULSEWIDTH	equ	500	; 250 us minimum pulse width
GAP		equ	1000	; 500 us between pulses
CENTERPOINT	equ	3040	; 1520 us center width
FULL_CYCLE	equ	40000	; 20 ms for all servos together

* ---+  +---------+  +---------+
*    |  |         |  |         |
*    |  |         |  |         |
*    |  |         |  |         |
*    +--+         +--+         +
*    500us
*       |<- svo 1  ->|
*        250-2000us
*

	org	MAIN_START
	
phase		fcb	0
variable_servo0	fdb	0	; servo motor periods
variable_servo1	fdb	0
variable_servo2	fdb	0
variable_servo3	fdb	0
variable_servo4	fdb	0
variable_servo5	fdb	0
recycle		fdb	0	; end of 20000 us pulsetrain

subroutine_initialize_module:
* init servo values to center point of travel
	ldx	#variable_servo0
	ldd	#CENTERPOINT-MIN_PULSEWIDTH
init_lp	std	0,x
	inx
	inx
	cpx	#recycle
	bne	init_lp
* init vector
	ldx	#toc3int_routine
	stx	TOC3INT
	rts	

toc3int_routine
	ldx	#REGBASE
	bclr	TFLG1,X $FF-$20	; clear interrupt flag
	
	ldaa	phase
	beq	phase0
	cmpa	#13
	beq	phase13
	lsra			; move 0 bit into carry & divide by 2
	bcs	phaseodd
phaseeven
* select proper servo
	ldaa	phase
	lsra
	bsr	setlatch
* set up rising edge 500 us from now
	ldd	TOC3,x
	addd	#GAP
	std	TOC3,x
	bset	TCTL1,x $30
	bra	increturn
phaseodd
* set up falling edge based on mx values
	ldab	phase
	andb	#$fe		; strip low bit
	ldx	#variable_servo0
	abx
	ldd	0,x		; load d from servo_x based on phase
	addd	#MIN_PULSEWIDTH	; add min pulse width
	ldx	#REGBASE
	addd	TOC3,X
	std	TOC3,X		; store time for fall edge
	bset	TCTL1,x $20
	bclr	TCTL1,x $10	; setup to fall
	bra	increturn
phase0
* select servo 0
	clra
	bsr	setlatch
* set up rising edge at 500 us and capture 20000 cycle time
	ldd	TOC3,x
	addd	#MIN_PULSEWIDTH
	std	TOC3,x
	bset	TCTL1,x $30
	addd	#FULL_CYCLE-MIN_PULSEWIDTH
	std	recycle
	bra	increturn
phase13
* set up falling edge at recycle time
	ldd	recycle
	std	TOC3,x
	bset	TCTL1,x $20	; setup to fall
	bclr	TCTL1,x $10
*	fall through to increturn	

increturn
	ldaa	phase
	inca
	cmpa	#14
	bne	incdone
	clra
incdone	staa	phase
	rti

* A contains number to write to latch
* servo selects are hooked up to low 3 bits
setlatch
	ldab	EXPVAL
	andb	#%11111000
	anda	#%00000111	; so our software wont break others
	aba
	staa	EXPLATCH
	staa	EXPVAL
	rts

subroutine_init_expbd_servos:
	aba
	beq	disable_expbd_servos

enable_expbd_servos
	clr	phase
	clra
	bsr	setlatch
	
	ldx	#REGBASE
	ldd	TCNT,x
	addd	#2000
	std	recycle
	std	TOC3,x
	bclr	TCTL1,x $10
	bset	TCTL1,x $20	; setup to clear
	
	bset	TMSK1,x $20
	rts	

disable_expbd_servos
	ldx	#$1000
	bclr	TMSK1,x $20
	bset	PORTA,x $20	
	rts

